#include "mainwindowimpl.hpp"

MainWindowImpl::MainWindowImpl(MainWindow* mw, ControlWindow *cw, ControlWindowImpl* cwi)
: QObject(), mw(mw), cw(cw), cwi(cwi) {}

//
// Speichert das Fraktal-Bild in niedriger Aufloesung mit 800 x 600 px.
// Rendering ist nicht notwendig, da das Fraktal-Bild im MainWindow hinterlegt wurde.
//
void MainWindowImpl::saveAsLow()
{
    if ((fileName = getFileName()) != "")
    {
        mw->fractalImage.save(fileName);
    }
}

//
// Speichert das Fraktal-Bild in mittlerer Aufloesung mit 1600 x 1200 px.
// Das Fraktal-Bild muss erst gerendert werden. Die Anwendung ist waehrenddessen blockiert.
//
void MainWindowImpl::saveAsMedium()
{
    if ((fileName = getFileName()) != "")
    {
        fractalImage = cwi->renderFractal(getCurrentFractal(), "", false, 2);
        fractalImage.save(fileName);
    }
}

//
// Speichert das Fraktal-Bild in hoher Aufloesung mit 3200 x 2400 px.
// Das Fraktal-Bild muss erst gerendert werden. Die Anwendung ist waehrenddessen blockiert.
//
void MainWindowImpl::saveAsHigh()
{
    if ((fileName = getFileName()) != "")
    {
        fractalImage = cwi->renderFractal(getCurrentFractal(), "", false, 4);
        fractalImage.save(fileName);
    }
}

//
// Anzeigen eines informativen Textes zur Mandelbrot-Menge.
//
void MainWindowImpl::mandelbrot()
{
    QMessageBox::question(mw,
                          tr("Mandelbrot-Menge"),
                          tr("Laut Wikipedia ist die Mandelbrot-Menge \"eine fraktal erscheinende "
                          "Menge, die eine bedeutende Rolle in der Chaosforschung spielt\".\n\n"
                          "Eine komplexe Zahl c ist Element der Menge, wenn die Folge\n\n"
                          "z(0) = 0\n"
                          "z(n+1) = z(n)^2 + c\n\n"
                          "beschraenkt ist."),
                          QMessageBox::Ok,
                          QMessageBox::Ok);
}

//
// Anzeigen eines informativen Textes zur Mandelbar-Menge.
//
void MainWindowImpl::mandelbar()
{
    QMessageBox::question(mw,
                          tr("Mandelbar-Menge"),
                          tr("Wandelt man die Berechnungsvorschrift der Mandelbrot-Menge so ab, "
                          "dass die Zahl z vor dem Quadrieren zunaechst konjugiert wird, ergibt "
                          "sich daraus eine als Mandelbar bezeichnete Menge.\n\n"
                          "Eine komplexe Zahl c ist Element der Menge, wenn die Folge\n\n"
                          "z(0) = 0\n"
                          "z(n+1) = conj(z)(n)^2 + c\n\n"
                          "beschraenkt ist."),
                          QMessageBox::Ok,
                          QMessageBox::Ok);
}

//
// Anzeigen eines informativen Textes zur Multibrot-Menge.
//
void MainWindowImpl::multibrot()
{
    QMessageBox::question(mw,
                          tr("Multibrot-Menge"),
                          tr("Die Berechnungsvorschrift der Mandelbrot-Menge kann dahingehend "
                          "verallgemeinert werden, dass der Exponent m frei waehlbar wird und "
                          "somit auch groessere Werte als 2 annehmen kann.\n\n"
                          "Eine komplexe Zahl c ist Element der Menge, wenn die Folge\n\n"
                          "z(0) = 0\n"
                          "z(n+1) = z(n)^m + c\n\n"
                          "beschraenkt ist."),
                          QMessageBox::Ok,
                          QMessageBox::Ok);

}

//
// Anzeigen eines informativen Textes zur Julia-Menge.
//
void MainWindowImpl::julia()
{
    QMessageBox::question(mw,
                          tr("Julia-Menge"),
                          tr("Zu jeder Funktion f : C -> C existiert eine Julia- und Fatou-"
                          "Menge. Durch wiederholte Anwendung von f - mit einem beliebigen "
                          "Startwert z - entsteht eine Folge komplexer Zahlen:\n\n"
                          "z -> f(z) -> f(f(z)) -> ...\n\n"
                          "Das Verhalten dieser Folge entscheidet darueber, ob der gewaehlte "
                          "Startwert ein Teil der Fatou- oder Julia-Menge ist.\n\n"
                          "Die in dieser Applikation verwendete Funktion f hat die Form "
                          "f(z) = z^2 + c, wobei c eine komplexe, frei waehlbare Zahl ist."),
                          QMessageBox::Ok,
                          QMessageBox::Ok);
}

//
// Anzeigen einiger weniger Hintergrundinformation zu diesem Projekt.
//
void MainWindowImpl::about()
{
    QMessageBox::question(mw,
                          tr("Ueber Fraktale"),
                          tr("Entwickelt von Marcell Dietl (www.marcell-dietl.de) im Januar 2012 "
                          "als Abschlussprojekt des Moduls \"Programmieren 3 AI\" (Java und C++) "
                          "des Studiengangs \"Allgemeine Informatik\" der Hochschule RheinMain "
                          "am Standort Wiesbaden in Hessen."),
                          QMessageBox::Ok,
                          QMessageBox::Ok);
}

//
// Oeffnen des Datei-Dialogs zum Speichern des Fraktal-Bildes.
//
QString MainWindowImpl::getFileName()
{
    return QFileDialog::getSaveFileName(mw, tr("Bild speichern unter..."), tr("Fraktal.png"));
}

//
// Ermitteln, welches Fraktal derzeit sichtbar ist. Wird benoetigt, um das Bild
// in mittlerer bzw. hoher Aufloesung zu rendern und anschlieend speichern zu koennen.
//
ComplexFractal* MainWindowImpl::getCurrentFractal()
{
    if (mw->fractalLabel->name == "mandelbar")
    {
        fractal = new Mandelbarmenge();
    }
    else if (mw->fractalLabel->name == "multibrot")
    {
        double exponent = cw->multibrotSpinBox->value();
        fractal = new Multibrotmenge();
        ((ParametrizedComplexFractal*)fractal)->setParam(std::complex<double>(exponent));
    }
    else if (mw->fractalLabel->name == "julia")
    {
        double real = cw->juliaRealSpinBox->value();
        double imag = cw->juliaImagSpinBox->value();
        fractal = new Juliamenge();
        ((ParametrizedComplexFractal*)fractal)->setParam(std::complex<double>(real, imag));
    }
    else
    {
        fractal = new Mandelbrotmenge();
    }

    return fractal;
}
